@@ -1,6 +1,5 @@
 /*
-**  pth_high.c -- Pth high-level replacement functions
-**
+**  GNU Pth - The GNU Portable Threads
 **  Copyright (c) 1999 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This file is part of GNU Pth, a non-preemptive thread scheduling
@@ -20,6 +19,8 @@
 **  License along with this library; if not, write to the Free Software
 **  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 **  USA, or contact Ralf S. Engelschall <rse@engelschall.com>.
+**
+**  pth_high.c: Pth high-level replacement functions
 */
                              /* ``The difference between a computer
                                   industry job and open-source software
@@ -594,6 +595,10 @@ ssize_t pth_write_ev(int fd, const void *buf, size_t nbytes, pth_event_t ev_extr
                 continue;
             }
 
+            /* pass error to caller, but not for partial writes (rv > 0) */
+            if (s < 0 && rv == 0)
+                rv = -1;
+
             /* stop looping */
             break;
         }
@@ -840,6 +845,10 @@ ssize_t pth_writev_ev(int fd, const struct iovec *iov, int iovcnt, pth_event_t e
                 continue;
             }
 
+            /* pass error to caller, but not for partial writes (rv > 0) */
+            if (s < 0 && rv == 0)
+                rv = -1;
+
             /* stop looping */
             break;
         }
